//*****************************************************************************************
// Truevision - a 3d modeler for gnome and povray
//
// scene.h
//
// Vincent LE PRINCE <vincentleprince@users.sourceforge.net>
// Copyright (C) 2000-2001 Vincent LE PRINCE
// This file is part of the TRUEVISION Package

//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
//*******************************************************************************************
#ifndef TV_SCENE_H
#define TV_SCENE_H
using namespace std;
#include "main.h"
#include "objectlist.h"
#include "matlist.h"

#define SCENE_DEF Scene *scene = (Scene*)(app_ref->scene);

//**********************************************
// Classe scene
//**********************************************
class Scene : public app_object {
	// Callbacks
	#define SENDER ((Scene*)data)
	friend void sign_export_to_povray( GtkWidget *wid, gpointer data ) { SENDER->export_to_povray_ask_fname(); }	
	friend void sign_del_filebox( GtkWidget *wid, gpointer data ) { gtk_widget_destroy( SENDER->filebox ); }	
	friend void sign_export_to_povray_with_fbox( GtkWidget *wid, gpointer data ) { SENDER->export_to_povray_with_fbox(); }	

	friend void sign_scene_save( GtkWidget *wid, gpointer data ) { SENDER->save(); }	
	friend void sign_scene_save_as( GtkWidget *wid, gpointer data ) { SENDER->save_as(); }	
   friend void sign_save_as_with_fbox( GtkWidget *wid, gpointer data ) { SENDER->save_as_with_fbox(); }		

	friend void sign_scene_new( GtkWidget *wid, gpointer data ) { SENDER->new_scene(); }
	
	friend void sign_scene_load( GtkWidget *wid, gpointer data ) { SENDER->load_as(); }	
	friend void sign_load_with_fbox( GtkWidget *wid, gpointer data ) { SENDER->load_with_fbox(); }		   	
	#undef SENDER
	
	
	private:
		app_objs *app_ref;
		char *filename;
		char *default_path;
		bool modified;
		ObjectList *objlist;
		TexList *texlist;
		GtkWidget *filebox;
		GtkWidget *tbw_save, *mew_save;
		int load_file_size;
		int save_file_size, save_file_state;

	private:
		void set_filename( const char *filename );
		void store_default_path( char *path );
		void save_widgets_active( gboolean var );
		

	public:
		Scene( app_objs *app_ref );

		void set_win_title();
		void set_modified();
		void set_save_tb_widget( GtkWidget *wid ) { tbw_save = wid; }
		void set_save_menu_widget( GtkWidget *wid ) { mew_save = wid; }
		bool is_modified() { return modified; }
		
		void output_to_povray( const char *filename );
		void export_to_povray_ask_fname();
		void export_to_povray_with_fbox();
		char *get_temp_povray_file();

		void save();
		void save_as();
		void save_as_with_fbox();
		void set_save_progress();

		
		void clear_scene();
		void new_scene();
		
		void load_as();
		void load_with_fbox();
		void set_load_progress( ifstream & file );
};

#endif

